// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "GameFramework/Character.h"
#include "PublicCharacter.generated.h"

UCLASS()
class SIMPLEGAME_API APublicCharacter : public ACharacter
{
	GENERATED_BODY()

public:
	// Sets default values for this character's properties
	APublicCharacter();
	
	USkeletalMeshComponent* GetSpecificPawnMesh() const;

	FName GetWeaponAttachPoint() const;

	void EquipWeapon(class ASimpleWeapon* Weapon);

	void OnChangeWeapon();

	void WeaponAttack();

	UPROPERTY(EditAnyWhere, BlueprintReadWrite, Category = Health)
		float Health;

	virtual float TakeDamage(float Damage, struct FDamageEvent const& DamageEvent, AController* EventInstigator, AActor* DamageCauser) override;

protected:
	UPROPERTY(EditDefaultsOnly, Category = Inventory)
	FName WeaponAttachPoint;

	TArray<class ASimpleWeapon*> Inventory;

	class ASimpleWeapon* CurrentWeapon;

	void AddWeapon(class ASimpleWeapon* Weapon);

	void SetCurrentWeapon(class ASimpleWeapon* NewWeapon, class ASimpleWeapon* LastWeapon);

	UPROPERTY(EditDefaultsOnly, Category = Inventory)
		TArray<TSubclassOf<class ASimpleWeapon>> DefaultInventoryClasses;

	void SpawnDefaultInventory();

	UPROPERTY(EditDefaultsOnly, Category = Pawn)
		UAnimMontage* BehitAnim;
	UPROPERTY(EditDefaultsOnly, Category = Pawn)
		UAnimMontage* DeathAnim;

	virtual void OnHit(float DamageTaken, struct FDamageEvent const& DamageEvent, class APawn* PawnInstigator, class AActor* DamageCauser);

	virtual void Die(float KillingDamage, struct FDamageEvent const& DamageEvent, AController* Killer, AActor* DamageCauser);

	void OnDieAnimationEnd();
};
