// Fill out your copyright notice in the Description page of Project Settings.

#include "SimpleGame.h"
#include "Engine.h"
#include "SimpleWeapon.h"


// Sets default values
ASimpleWeapon::ASimpleWeapon(const class FObjectInitializer& ObjectInitializer)
	:Super(ObjectInitializer)
{
 	// Set this actor to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = true;

	WeaponMesh = ObjectInitializer.CreateDefaultSubobject<USkeletalMeshComponent>(this, TEXT("WeaponMesh"));
	WeaponMesh->CastShadow = true;
	WeaponMesh->SetCollisionEnabled(ECollisionEnabled::NoCollision);
	RootComponent = WeaponMesh;

	WeaponCollision = CreateDefaultSubobject<USphereComponent>(TEXT("WeaponCollision"));
	WeaponCollision->InitSphereRadius(30.0f);
	WeaponCollision->AttachTo(WeaponMesh, "DamageSocket");

	DuringAttack = false;
}

void ASimpleWeapon::SetOwningPawn(APublicCharacter* NewOwner)
{
	if (MyPawn != NewOwner)
	{
		MyPawn = NewOwner;
	}
}

void ASimpleWeapon::AttachMeshToPawn()
{
	if (MyPawn)
	{
		USkeletalMeshComponent* PawnMesh = MyPawn->GetSpecificPawnMesh();
		FName AttachPoint = MyPawn->GetWeaponAttachPoint();
		WeaponMesh->AttachTo(PawnMesh, AttachPoint);
	}
}

void ASimpleWeapon::OnEquip(const ASimpleWeapon* LastWeapon)
{
	AttachMeshToPawn();
}

void ASimpleWeapon::OnUnEquip()
{
	WeaponMesh->DetachFromParent();
}

void ASimpleWeapon::StartAttack()
{
	float AnimDuration = MyPawn->PlayAnimMontage(AttackAnim);

	DuringAttack = true;

	FTimerHandle TimerHandle_StopAttack;
	GetWorldTimerManager().SetTimer(TimerHandle_StopAttack, this, &ASimpleWeapon::StopAttack, AnimDuration, false);
}

void ASimpleWeapon::StopAttack()
{
	MyPawn->StopAnimMontage(AttackAnim);
	DuringAttack = false;
}

void ASimpleWeapon::NotifyActorBeginOverlap(AActor* OtherActor)
{
	Super::NotifyActorBeginOverlap(OtherActor);

	if (OtherActor->IsA(AActor::StaticClass()) && DuringAttack == true)
	{
		UGameplayStatics::ApplyDamage(OtherActor, 50.0f, NULL, this, UDamageType::StaticClass());

		GEngine->AddOnScreenDebugMessage(-1, 2.0f, FColor::Black, "ApplyDamage!!!");
	}
}


